#pragma version(1)
#pragma rs java_package_name(cn.wps.show)

uint8_t *grayArray;
rs_allocation gIn;
rs_allocation gOut;

const int32_t GRAY_COUNT = 256;
uint32_t histogram[GRAY_COUNT];
uint8_t threshold;
int32_t width;
int32_t height;

static uint8_t getGray(uchar3 rgb) {
    return rgb.r * 0.3f + rgb.g * 0.59f + rgb.b * 0.11f;
}

static void setup() {
    width = rsAllocationGetDimX(gIn);
    height = rsAllocationGetDimY(gIn);
    for (int32_t i = 0; i < GRAY_COUNT; i++) {
        histogram[i] = 0;
    }

    int32_t index = 0;
    int32_t backgroundGraySum = 0;
    uint8_t maxGray = 0;
    uint8_t minGray = 255;
    uint8_t bgGray = 0;
    for (int32_t y = 0; y < height; y++) {
        for (int32_t x = 0; x < width; x++) {
            uchar4 pixel = *(const uchar4*)rsGetElementAt(gIn, x, y);

            uint8_t gray = getGray(pixel.rgb);
            histogram[gray]++;
            grayArray[index] = gray;

            if (maxGray < gray) {
                maxGray = gray;
            }
            if (minGray > gray) {
                minGray = gray;
            }
            if (y == 0 || y == height - 1 || x == 0 || x == width - 1) {
                backgroundGraySum += gray;
            }

            index++;
        }
    }
    bgGray = backgroundGraySum / ((height + width) * 2 - 4);

    threshold = 0;
    int32_t count1 = 0;
    int32_t graySum1 = 0;
    int32_t total = width * height;
    float tempG = -1;
    for (int32_t i = 0; i < 256; i++) {
        count1 += histogram[i];
        int count2 = total - count1;
        if (count1 == 0 || count2 == 0) {
            continue;
        }

        graySum1 += histogram[i] * i;
        float u1 = graySum1 * 1.0f / count1;

        int graySum2 = 0;
        for (int j = i + 1; j < 256; j++) {
            graySum2 += histogram[j] * j;
        }
        float u2 = graySum2 * 1.0f / count2;

        float w1 = count1 * 1.0f / total;
        float g = w1 * (1 - w1) * (u1 - u2) * (u1 - u2);
        if (g > tempG) {
            threshold = i;
            tempG = g;
        }
    }

    bool flag = bgGray > threshold;
    uint8_t m = flag ? 0 : threshold;
    uint8_t n = flag ? threshold : GRAY_COUNT - 1;
    uint8_t maxFGGray = m;
    uint32_t maxFGGrayCount = histogram[m];
    for (uint32_t i = m + 1; i <= n; i++) {
        uint32_t grayCount = histogram[i];
        if (grayCount > maxFGGrayCount) {
            maxFGGray = i;
            maxFGGrayCount = grayCount;
        }
    }

    index = 0;
    float4 total4;
    total4.a = 0;
    total4.r = 0;
    total4.g = 0;
    total4.b = 0;
    int32_t count = 0;
    for (int32_t y = 0; y < height; y++) {
        for (int32_t x = 0; x < width; x++) {
            if (grayArray[index] == maxFGGray) {
                float4 pixel = rsUnpackColor8888(*(const uchar4*)rsGetElementAt(gIn, x, y));
                total4.r += pixel.r;
                total4.g += pixel.g;
                total4.b += pixel.b;
                count++;
            }

            index++;
        }
    }

    total4.a = 1;
    if (count != 0) {
        total4.r /= count;
        total4.g /= count;
        total4.b /= count;
    }
    uchar4 result = rsPackColorTo8888(total4);
    rsDebug("color", result);
    rsSetElementAt_uchar4(gOut, result, 0);
}

void start() {
    setup();
    // rs
}